import requests
import pandas as pd
from datetime import datetime
import re

# 你的 Threads API Access Token 與 User ID
ACCESS_TOKEN = '替換成你的Threads API Access Token'
USER_ID = '替換成你的USER ID'

BASE_URL = 'https://graph.threads.net/v1.0'
METRICS = ['likes', 'replies', 'views', 'reposts', 'quotes', 'shares']

def format_timestamp(iso_timestamp):
    """
    將 ISO 8601 格式的時間轉換為 YYYY/MM/DD HH:MM 格式
    """
    if not iso_timestamp:
        return ""
    try:
        dt = datetime.strptime(iso_timestamp, "%Y-%m-%dT%H:%M:%S%z")
        # 轉換為台灣時區 (UTC+8)
        dt = dt.astimezone()
        return dt.strftime("%Y/%m/%d %H:%M")
    except:
        return ""

def get_current_time():
    """
    取得當前時間並格式化
    回傳：
    - display_time: 用於顯示的時間格式 (YYYY/MM/DD HH:MM)
    - file_time: 用於檔案名稱的時間格式 (YYYYMMDDHHMM)
    """
    now = datetime.now()
    display_time = now.strftime("%Y/%m/%d %H:%M")
    file_time = now.strftime("%Y%m%d%H%M")
    return display_time, file_time

def get_all_threads(user_id, access_token, limit=None):
    """
    取得 Threads 貼文 ID
    limit: 要抓取的貼文數量，None 表示抓取全部
    """
    url = f'{BASE_URL}/{user_id}/threads'
    if limit:
        url += f'?limit={limit}'
    
    headers = {'Authorization': f'Bearer {access_token}'}
    thread_ids = []

    while url:
        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            print(f'❌ 取得 threads 失敗：{response.status_code}')
            break
        result = response.json()
        for item in result.get("data", []):
            thread_ids.append(item["id"])
            # 如果已經達到指定的數量，就停止
            if limit and len(thread_ids) >= limit:
                break

        if limit and len(thread_ids) >= limit:
            break

        # 處理下一頁
        url = result.get("paging", {}).get("next", None)

    print(f'✅ 共取得 {len(thread_ids)} 篇貼文')
    return thread_ids

def get_thread_text(thread_id, access_token):
    url = f"{BASE_URL}/{thread_id}?fields=text,timestamp"
    headers = {'Authorization': f'Bearer {access_token}'}
    try:
        res = requests.get(url, headers=headers)
        if res.status_code == 200:
            data = res.json()
            return data.get("text", ""), format_timestamp(data.get("timestamp", ""))
        else:
            return "", ""
    except Exception:
        return "", ""

def get_thread_insights(thread_id, access_token, metrics):
    url = f"{BASE_URL}/{thread_id}/insights?metric={','.join(metrics)}"
    headers = {'Authorization': f'Bearer {access_token}'}
    try:
        res = requests.get(url, headers=headers)
        if res.status_code == 200:
            data = res.json().get("data", [])
            result = {m: 0 for m in metrics}
            for item in data:
                result[item["name"]] = item["values"][0]["value"]
            return result, "success"
        else:
            return {m: None for m in metrics}, f"error {res.status_code}"
    except Exception as e:
        return {m: None for m in metrics}, str(e)

def main():
    # 取得當前時間
    display_time, file_time = get_current_time()

    # 詢問要抓取多少篇貼文
    while True:
        try:
            limit = input("請輸入要抓取的貼文數量（直接按 Enter 表示抓取全部）：")
            if not limit:
                limit = None
            else:
                limit = int(limit)
                if limit <= 0:
                    print("❌ 貼文數量必須大於 0")
                    continue
            break
        except ValueError:
            print("❌ 請輸入有效的數字")
            continue

    thread_ids = get_all_threads(USER_ID, ACCESS_TOKEN, limit)
    data = []

    for i, tid in enumerate(thread_ids):
        print(f"📄 處理第 {i+1} 篇：{tid}")
        row = {"thread_id": tid}

        text, timestamp = get_thread_text(tid, ACCESS_TOKEN)
        row["text"] = text
        row["發布時間"] = timestamp

        insights, status = get_thread_insights(tid, ACCESS_TOKEN, METRICS)
        row.update(insights)
        row["status"] = status
        row["抓取時間"] = display_time

        data.append(row)

    df = pd.DataFrame(data)

    # 計算新的欄位
    df['總互動數'] = df['likes'] + df['replies'] + df['reposts'] + df['quotes'] + df['shares']
    
    # 計算互動率（當 views 為 0 時顯示空白）
    df['互動率'] = df.apply(lambda x: (x['likes'] + x['replies'] + x['reposts'] + x['quotes'] + x['shares']) / x['views'] 
                           if x['views'] > 0 else None, axis=1)
    
    # 計算深入互動數
    df['深入互動數'] = df['reposts'] + df['quotes'] + df['shares']
    
    # 計算深入互動率（當總互動數為 0 時顯示空白）
    df['深入互動率'] = df.apply(lambda x: (x['reposts'] + x['quotes'] + x['shares']) / x['總互動數'] 
                           if x['總互動數'] > 0 else None, axis=1)

    # 調整欄位順序
    ordered_columns = ['thread_id', 'text', '發布時間', 'views', 'likes', 'replies', 'reposts', 'quotes', 'shares', 
                      '總互動數', '互動率', '深入互動數', '深入互動率', 'status', '抓取時間']
    df = df.reindex(columns=ordered_columns)

    # 產生包含時間戳記的檔案名稱
    filename = f"threads_insights_{file_time}.xlsx"
    
    # 設定 Excel 寫入選項，確保空值顯示為空白
    with pd.ExcelWriter(filename, engine='openpyxl') as writer:
        df.to_excel(writer, index=False)
        # 設定百分比格式
        worksheet = writer.sheets['Sheet1']
        for col in ['互動率', '深入互動率']:
            col_idx = ordered_columns.index(col) + 1  # +1 因為 Excel 是 1-based
            for row in range(2, len(df) + 2):  # +2 因為 Excel 是 1-based 且跳過標題列
                cell = worksheet.cell(row=row, column=col_idx)
                if cell.value is not None:
                    cell.number_format = '0.0%'
                else:
                    cell.value = ''
    
    print(f"✅ 已匯出 Excel 檔案：{filename}")

if __name__ == "__main__":
    main()
